# frozen_string_literal: true

require_dependency 'jwt_api_auth/application_controller'

module JwtApiAuth
  class AuthenticationController < ApplicationController
    skip_before_action :authenticate_user, only: [:create]

    rescue_from ActionController::ParameterMissing do
      head :not_found
    end

    attr_reader :resource
    before_action :find_resource, only: [:create]
    before_action :authenticate, only: [:create]

    def create
      create_refresh_token if refresh_token_enabled?

      render json: { token: token }, status: :created
    end

    private

    def create_refresh_token
      refresh_token = refresh_token_model_class.create!(
        user_id: resource.id,
        token: SecureRandom.base64
      )

      cookies[:refreshToken] = {
        value: refresh_token.token,
        expires: JwtApiAuth.refresh_token_lifetime.from_now,
        httponly: true,
        path: '/refresh_token'
      }
    end

    def find_resource
      @resource = resource_class.find_by(email: email)
      head :not_found unless @resource
    end

    def email
      params.require(:email)
    end

    def password
      params.require(:password)
    end

    def authenticate
      head(:not_found) unless resource.authenticate(password)
    end

    def payload
      {
        sub: resource.id,
        aud: JwtApiAuth.token_audience
      }.delete_if { |_key, value| value.blank? }
    end

    def token
      ::JWT.encode payload, JwtApiAuth.token_secret.call, 'HS256'
    end
  end
end
